@@ -68,12 +68,13 @@ ZEND_BEGIN_ARG_INFO_EX(arginfo_zstd_uncompress, 0, 0, 1)
     ZEND_ARG_INFO(0, data)
 ZEND_END_ARG_INFO()
 
-ZEND_BEGIN_ARG_INFO_EX(arginfo_zstd_compress_dict, 0, 0, 1)
+ZEND_BEGIN_ARG_INFO_EX(arginfo_zstd_compress_dict, 0, 0, 2)
     ZEND_ARG_INFO(0, data)
     ZEND_ARG_INFO(0, dictBuffer)
+    ZEND_ARG_INFO(0, level)
 ZEND_END_ARG_INFO()
 
-ZEND_BEGIN_ARG_INFO_EX(arginfo_zstd_uncompress_dict, 0, 0, 1)
+ZEND_BEGIN_ARG_INFO_EX(arginfo_zstd_uncompress_dict, 0, 0, 2)
     ZEND_ARG_INFO(0, data)
     ZEND_ARG_INFO(0, dictBuffer)
 ZEND_END_ARG_INFO()
@@ -107,12 +108,6 @@ ZEND_FUNCTION(zstd_compress)
                  " must be within 1..%d", level, maxLevel);
 #endif
       RETURN_FALSE;
-    } else if (level == 0) {
-#if ZEND_MODULE_API_NO >= 20141001
-      RETURN_STRINGL(Z_STRVAL_P(data), Z_STRLEN_P(data));
-#else
-      RETURN_STRINGL(Z_STRVAL_P(data), Z_STRLEN_P(data), 1);
-#endif
     }
 
     size = ZSTD_compressBound(Z_STRLEN_P(data));
@@ -249,9 +244,11 @@ ZEND_FUNCTION(zstd_uncompress)
 ZEND_FUNCTION(zstd_compress_dict)
 {
     zval *data, *dictBuffer;
+    long level = DEFAULT_COMPRESS_LEVEL;
+    uint16_t maxLevel = (uint16_t) ZSTD_maxCLevel();
 
     if (zend_parse_parameters(ZEND_NUM_ARGS() TSRMLS_CC,
-                              "z|z", &data, &dictBuffer) == FAILURE) {
+                              "zz|l", &data, &dictBuffer, &level) == FAILURE) {
         RETURN_FALSE;
     }
     if (Z_TYPE_P(data) != IS_STRING) {
@@ -265,6 +262,18 @@ ZEND_FUNCTION(zstd_compress_dict)
         RETURN_FALSE;
     }
 
+#if ZSTD_VERSION_NUMBER >= 10304
+    if (level > maxLevel) {
+      zend_error(E_WARNING, "zstd_compress_dict: compression level (%ld)"
+                 " must be within 1..%d or smaller then 0", level, maxLevel);
+#else
+    if (level > maxLevel || level < 0) {
+      zend_error(E_WARNING, "zstd_compress_dict: compression level (%ld)"
+                 " must be within 1..%d", level, maxLevel);
+#endif
+      RETURN_FALSE;
+    }
+
     size_t const cBuffSize = ZSTD_compressBound(Z_STRLEN_P(data));
     void* const cBuff = emalloc(cBuffSize);
     if (!cBuff) {
@@ -279,7 +288,7 @@ ZEND_FUNCTION(zstd_compress_dict)
     }
     ZSTD_CDict* const cdict = ZSTD_createCDict(Z_STRVAL_P(dictBuffer),
                                                Z_STRLEN_P(dictBuffer),
-                                               DEFAULT_COMPRESS_LEVEL);
+                                               level);
     if (!cdict) {
         efree(cBuff);
         zend_error(E_WARNING, "ZSTD_createCDict() error");
@@ -312,7 +321,7 @@ ZEND_FUNCTION(zstd_uncompress_dict)
     zval *data, *dictBuffer;
 
     if (zend_parse_parameters(ZEND_NUM_ARGS() TSRMLS_CC,
-                              "z|z", &data, &dictBuffer) == FAILURE) {
+                              "zz", &data, &dictBuffer) == FAILURE) {
         RETURN_FALSE;
     }
     if (Z_TYPE_P(data) != IS_STRING) {
@@ -329,6 +338,8 @@ ZEND_FUNCTION(zstd_uncompress_dict)
     unsigned long long const rSize = ZSTD_getDecompressedSize(Z_STRVAL_P(data),
                                                               Z_STRLEN_P(data));
     if (rSize == 0) {
+        zend_error(E_WARNING, "zstd_uncompress_dict:"
+                   " it was not compressed by zstd");
         RETURN_FALSE;
     }
     void* const rBuff = emalloc((size_t)rSize);
